# -*- coding: utf-8 -*-
import re
import psutil

from amplify.agent.util import subp
from amplify.agent.context import context
from amplify.agent.containers.abstract import AbstractCollector


__author__ = "Mike Belov"
__copyright__ = "Copyright (C) 2015, Nginx Inc. All rights reserved."
__credits__ = ["Mike Belov", "Andrei Belov", "Ivan Poluyanov", "Oleg Mamontov", "Andrew Alexeev"]
__license__ = ""
__maintainer__ = "Mike Belov"
__email__ = "dedm@nginx.com"


class NginxMetaCollector(AbstractCollector):

    short_name = 'nginx_meta'

    def collect(self):
        meta = {
            'pid': self.object.pid,
            'local_id': self.object.local_id,
            'start_time': None,
            'running': True,
            'stub_status_enabled': self.object.stub_status_enabled,
            'status_module_enabled': self.object.plus_status_enabled,
            'stub_status_url': self.object.stub_status_url,
            'plus_status_url': self.object.plus_status_url,
            'version': None,
            'plus': {
                'enabled': False,
                'release': None
            },
            'configure': {},
            'packages': {},
            'path': {
                'bin': self.object.bin_path,
                'conf': self.object.conf_path,
                'log': {}
            },
            'warnings': [],
            'ssl': {},
        }

        for method in (
            self.find_installed_packages,
            self.nginx_minus_v,
            self.find_package_via_binary,
            self.nginx_uptime,
            self.open_ssl
        ):
            try:
                method(meta)
            except Exception, e:
                exception_name = e.__class__.__name__
                context.log.error('failed to collect meta %s due to %s' % (method.__name__, exception_name))
                context.log.debug('additional info:', exc_info=True)

        self.metad.meta(meta)

    @staticmethod
    def find_installed_packages(meta):
        """ trying to find some installed packages """
        dpkg_grep_nginx_out, _ = subp.call("dpkg -l | grep nginx")
        for line in dpkg_grep_nginx_out:
            gwe = re.match('([\d\w]+)\s+([\d\w\.\-]+)\s+([\d\w\.\-]+)\s', line)
            if gwe:
                if gwe.group(2).startswith('nginx'):
                    meta['packages'][gwe.group(2)] = gwe.group(3)

    def nginx_minus_v(self, meta):
        """ call -V and parse """
        _, nginx_v_err = subp.call("%s -V" % self.object.bin_path)
        for line in nginx_v_err:
            # SSL stuff
            if line.lower().startswith('built with') and 'ssl' in line.lower():
                parts = line.split(' ')
                lib_name, lib_version = parts[2:4]
                meta['ssl'] = {
                    'built': [lib_name, lib_version],
                    'run': [lib_name, lib_version],
                }
            if line.lower().startswith('run with') and 'ssl' in line.lower():
                parts = line.split(' ')
                lib_name, lib_version = parts[2:4]
                meta['ssl']['run'] = [lib_name, lib_version]

            parts = line.split(':')
            if len(parts) < 2:
                continue

            # parse version
            key, value = parts
            if key == 'nginx version':
                # parse major version
                major_parsed = re.match('.*/([\d\w\.]+)', value)
                meta['version'] = major_parsed.group(1) if major_parsed else value.lstrip()

                # parse plus version
                if 'plus' in value:
                    plus_parsed = re.match('.*\(([\w\-]+)\).*', value)
                    if plus_parsed:
                        meta['plus']['enabled'] = True
                        meta['plus']['release'] = plus_parsed.group(1)

            # parse configure
            if key == 'configure arguments':
                arguments = self._parse_arguments(value)
                meta['configure'] = arguments

                if 'conf-path' in arguments and not self.object.conf_path:
                    meta['path']['conf'] = arguments['conf-path']
                    self.object.conf_path = arguments['conf-path']
                if 'http-log-path' in arguments:
                    meta['path']['log']['http'] = arguments['http-log-path']
                if 'error-log-path' in arguments:
                    meta['path']['log']['error'] = arguments['error-log-path']

    def find_package_via_binary(self, meta):
        """ find a package with running binary """
        package_name = None

        dpkg_s_nginx_out, dpkg_s_nginx_err = subp.call('dpkg -S %s' % self.object.bin_path, force=True)
        for line in dpkg_s_nginx_out:
            kv = re.match('([\w\-\.]+)\s*:\s*(.+)', line)
            if kv:
                package_name = kv.group(1)
                break

        if dpkg_s_nginx_err:
            if 'no_path' in dpkg_s_nginx_err[0]:
                meta['warnings'].append('self-made binary, is not from any nginx package')

        if not package_name:
            return

        # packages mismatch
        if package_name not in meta['packages']:
            meta['warnings'].append('binary is not from standard nginx packages')

        # if found then get all the information about the package
        apt_cache_out, _ = subp.call('apt-cache policy %s'% package_name, force=True)
        for line in apt_cache_out:
            if 'Installed' in line:
                meta['packages'][package_name] = line.split(': ')[1]

    def nginx_uptime(self, meta):
        # collect info about start time
        master_process = psutil.Process(self.object.pid)
        meta['start_time'] = int(master_process.create_time()) * 1000

    def open_ssl(self, meta):
        """Old nginx uses standart openssl library - find its version"""
        if not meta['ssl']:
            openssl_out, _ = subp.call("dpkg -l | grep openssl")
            for line in openssl_out:
                gwe = re.match('([\d\w]+)\s+([\d\w\.\-]+)\s+([\d\w\.\-\+_~]+)\s', line)
                if gwe:
                    if gwe.group(2).startswith('openssl'):
                        meta['ssl'] = {
                            'built': [gwe.group(2), gwe.group(3)],
                            'run': [gwe.group(2), gwe.group(3)],
                        }

    @staticmethod
    def _parse_arguments(argstring):
        arguments = {}

        current_key = None
        current_value = None

        for part in argstring.split(' --'):
            if '=' in part:
                # next part of compound
                if current_key and current_value:
                    current_value += part
                    if part.endswith("'"):
                        arguments[current_key] = current_value
                        current_key = None
                        current_value = None
                else:
                    k, v = part.split('=', 1)
                    # compound argument
                    if v.startswith("'") and v.endswith("'"):
                        arguments[k] = v
                    elif v.startswith("'"):
                        current_key = k
                        current_value = v
                    # simple argument
                    else:
                        arguments[k] = v
            else:
                # boolean
                if part:
                    arguments[part] = True

        return arguments
